include graphics.e
include rect.e
include get.e
include font.e

-- Global constants
constant BlockSize = 30,
         FlagOn    = 7,
         FlagOff   = 0,
         NumChars  = "12345678"

-- Global variables
sequence Blocks,Flags,TheFont
atom curchar

-- function to convert a char value to an integer value.
function AtoI(atom C)
  integer i,pos
  pos = find(C,NumChars)
  i = 0
  if pos > 0 then
    i = pos
  end if
  return i
end function

procedure StartGraphics(atom mode)
  atom junk
  junk = graphics_mode(mode)
end procedure

procedure InitFlags()
  Flags = repeat(0,8)
  Flags = repeat(Flags,8)
end procedure

procedure InitBlocks()
  atom i
  sequence pos
  Blocks = repeat(0,8)
  Blocks = repeat(Blocks,8)
  i = 1
  for y = 1 to 8 do
    for x = 1 to 8 do
      pos = {x,y}-1
      Blocks[y][x] = new_rect(pos*BlockSize,pos*BlockSize+BlockSize)
      polygon(15,0,Blocks[y][x])
      i = i + 1
    end for
  end for
end procedure

procedure DrawFlags()
  for y = 1 to 8 do
    for x = 1 to 8 do
      polygon(Flags[y][x],1,Blocks[y][x])
      polygon(15,0,Blocks[y][x])
    end for
  end for
end procedure

procedure ToggleFlag(sequence Flag)
  atom x,y
  x = Flag[1]
  y = Flag[2]

  if Flags[y][x] = FlagOn then
    Flags[y][x] = FlagOff
    polygon(0,1,Blocks[y][x])
    polygon(15,0,Blocks[y][x])
  else
    Flags[y][x] = FlagOn
    polygon(7,1,Blocks[y][x])
    polygon(15,0,Blocks[y][x])
  end if
end procedure

procedure DrawCommands()
  atom x
  x = BlockSize*8+5
  outtextxy(15,{x,0},"- commands -")
  outtextxy(7,{x,10},"t - toggle flag on/off")
  outtextxy(7,{x,20},"c - get character")
  outtextxy(7,{x,30},"s - save character to the font")
  outtextxy(7,{x,40},"f - save font to disk")
  outtextxy(7,{x,50},"l - load font from disk")
  outtextxy(7,{x,60},"q - quit the font editor")

end procedure

procedure GetChar(atom char)
  Flags = TheFont[char+1]
  DrawFlags()
  curchar = char
end procedure

procedure Init()

  printf(1,"Font Editor for FONT.E by NoSolution.\n",{})
  printf(1,"Loading Font BIOS.FNT ... ",{})
  LoadFont("bios.fnt")
  TheFont = Font
  printf(1,"done\nPress any key...\n",{})

  while get_key() = -1 do end while

  StartGraphics(18)
  InitBlocks()
  InitFlags()
  DrawCommands()
  curchar = 0
end procedure

procedure Main()
  atom cmd
  atom x,y,a,b
  sequence s,fname
  Init()

  cmd = 0
  s = "_"
  while cmd != 'q' do
    cmd = get_key()
    if cmd = 't' then
      s = repeat(0,80)
      outtextxy(15,{0,BlockSize*8+5},s)
      outtextxy(15,{0,BlockSize*8+5},"toggle(")
      a = wait_key()
      while a < '1' or a > '8' do
        a = wait_key()
      end while
      x = AtoI(a)
      s[1] = a
      outtextxy(15,{56,BlockSize*8+5}, s)
      outtextxy(15,{64,BlockSize*8+5}, ",")
      b = wait_key()
      while b < '1' or b > '8' do
        b = wait_key()
      end while
      y = AtoI(b)
      s[1] = b
      outtextxy(15,{72,BlockSize*8+5}, s)
      outtextxy(15,{80,BlockSize*8+5}, ")")
      ToggleFlag({x,y})
    end if
    if cmd = 'c' then
      s = repeat(0,80)
      outtextxy(15,{0,BlockSize*8+5},s)
      outtextxy(15,{0,BlockSize*8+5},"char(")
      a = wait_key()
      outtextxy(15,{40,BlockSize*8+5},'"'&a&'"'&')')
      GetChar(a)
    end if
    if cmd = 's' then
      s = repeat(0,80)
      outtextxy(15,{0,BlockSize*8+5},s)
      outtextxy(15,{0,BlockSize*8+5},"Save() Are you Sure?(y/n)")
      a = wait_key()
      if a = 'y' then
        TheFont[curchar+1] = Flags
      end if
      s = repeat(0,80)
      outtextxy(15,{0,BlockSize*8+5},s)
    end if
    if cmd = 'f' then
      s = repeat(0,80)
      outtextxy(15,{0,BlockSize*8+5},s)
      outtextxy(15,{0,BlockSize*8+5},"SaveFile(\"")
      fname = intextxy(15,{80,BlockSize*8+5})
      outtextxy(15,{80+(length(fname)*8),BlockSize*8+5},"\")")
      Font = TheFont
      SaveFont(fname)
      s = repeat(0,80)
      outtextxy(15,{0,BlockSize*8+5},s)
      outtextxy(15,{0,BlockSize*8+5},"Font saved.")
    end if
    if cmd = 'l' then
      s = repeat(0,80)
      outtextxy(15,{0,BlockSize*8+5},s)
      outtextxy(15,{0,BlockSize*8+5},"LoadFile(\"")
      fname = intextxy(15,{80,BlockSize*8+5})
      outtextxy(15,{80+(length(fname)*8),BlockSize*8+5},"\")")
      
      s = repeat(0,80)
      outtextxy(15,{0,BlockSize*8+5},s)
      outtextxy(15,{0,BlockSize*8+5},"Loading font")
      LoadFont(fname)
      TheFont = Font
      s = repeat(0,80)
      outtextxy(15,{0,BlockSize*8+5},s)
      outtextxy(15,{0,BlockSize*8+5},"Done")

      
    end if

  end while
    
  StartGraphics(-1)

end procedure


Main()




      




